@@ -2,9 +2,11 @@
 
 __metaclass__ = type
 
+import gc
 from zope.interface import implements, Interface
 
 from inspect import getabsfile
+from weakref import WeakValueDictionary
 
 from twisted.python import log
 from twisted.python.reflect import qual, namedAny
@@ -21,7 +23,7 @@ from axiom.attributes import (
     _OrderingMixin, _ContainableMixin, Comparable, compare, inmemory,
     reference, text, integer, AND, _cascadingDeletes, _disallows)
 
-_typeNameToMostRecentClass = {}
+_typeNameToMostRecentClass = WeakValueDictionary()
 
 def normalize(qualName):
     """
@@ -60,6 +62,9 @@ class MetaItem(slotmachine.SchemaMetaMachine):
             T.typeName = normalize(qual(T))
         if T.schemaVersion is None:
             T.schemaVersion = 1
+        if T.typeName in _typeNameToMostRecentClass:
+            # Let's try not to gc.collect() every time.
+            gc.collect()
         if T.typeName in _typeNameToMostRecentClass:
             if T.__legacy__:
                 return T
@@ -74,8 +79,10 @@ class MetaItem(slotmachine.SchemaMetaMachine):
                 else:
                     relmod = otherT.__module__
 
-                raise RuntimeError("Use absolute imports; relative import"
-                        " detected for type %r (imported from %r)" % (T.typeName, relmod))
+                raise RuntimeError(
+                    "Use absolute imports; relative import"
+                    " detected for type %r (imported from %r)" % (
+                        T.typeName, relmod))
 
             raise RuntimeError("2 definitions of axiom typename %r: %r %r" % (
                     T.typeName, T, _typeNameToMostRecentClass[T.typeName]))
@@ -90,7 +97,8 @@ class MetaItem(slotmachine.SchemaMetaMachine):
         purposes.
         """
         if isinstance(other, MetaItem):
-            return cmp(self.typeName, other.typeName)
+            return cmp((self.typeName, self.schemaVersion),
+                       (other.typeName, other.schemaVersion))
         return NotImplemented
 
 
@@ -387,7 +395,7 @@ class Empowered(object):
                 pifs = [(i, p) for (i, p) in pifs]
             except ValueError:
                 raise ValueError("return value from %r.__getPowerupInterfaces__"
-                                 "not an iterable of 2-tuples" % (obj,))
+                                 " not an iterable of 2-tuples" % (self,))
         return pifs
 
 
@@ -651,7 +659,7 @@ class Item(Empowered, slotmachine._Strict):
                          storeID=storeID,
                          __everInserted=True)
 
-        schema = list(self.getSchema())
+        schema = self.getSchema()
         assert len(schema) == len(attrs), "invalid number of attributes"
         for data, (name, attr) in zip(attrs, schema):
             attr.loaded(self, data)
@@ -667,11 +675,13 @@ class Item(Empowered, slotmachine._Strict):
         """
         return all persistent class attributes
         """
+        schema = []
         for name, atr in cls.__attributes__:
             atr = atr.__get__(None, cls)
             if isinstance(atr, SQLAttribute):
-                yield (name, atr)
-
+                schema.append((name, atr))
+        cls.getSchema = staticmethod(lambda schema=schema: schema)
+        return schema
     getSchema = classmethod(getSchema)
 
 
@@ -1061,9 +1071,14 @@ class Placeholder(object):
         # once in the "SELECT ..." part of the query, determined by
         # getSchema(). In this case, the correct placeholder names
         # need to be used.
+        schema = []
         for (name, atr) in self._placeholderItemClass.getSchema():
-            yield name, _PlaceholderColumn(self,
-                                           getattr(self._placeholderItemClass, name))
+            schema.append((
+                    name,
+                    _PlaceholderColumn(
+                        self, getattr(self._placeholderItemClass, name))))
+        return schema
+
 
     def getTableName(self, store):
         return self._placeholderItemClass.getTableName(store)
