@@ -1,12 +1,9 @@
 
 from twisted.trial import unittest
-from twisted.python import log, failure, filepath
+from twisted.python import failure, filepath
 from twisted.application import service
 
 from axiom import iaxiom, store, item, attributes, batch, substore
-from axiom.scheduler import Scheduler
-
-from axiom.dependency import installOn
 
 class TestWorkUnit(item.Item):
     information = attributes.integer()
@@ -41,8 +38,7 @@ class BatchTestCase(unittest.TestCase):
     def setUp(self):
         self.procType = batch.processor(TestWorkUnit)
         self.store = store.Store()
-        self.scheduler = Scheduler(store=self.store)
-        installOn(self.scheduler, self.store)
+        self.scheduler = iaxiom.IScheduler(self.store)
 
 
     def testItemTypeCreation(self):
@@ -230,7 +226,7 @@ class BatchTestCase(unittest.TestCase):
         self.assertEquals(errors[0][0], listener)
         self.assertEquals(errors[0][1].information, 1)
 
-        loggedErrors = log.flushErrors(RuntimeError)
+        loggedErrors = self.flushLoggedErrors(RuntimeError)
         self.assertEquals(len(loggedErrors), 1)
         self.assertEquals(loggedErrors[0].getErrorMessage(), errmsg)
 
@@ -584,7 +580,6 @@ class RemoteTestCase(unittest.TestCase):
 
         dbdir = filepath.FilePath(self.mktemp())
         st = store.Store(dbdir)
-        installOn(Scheduler(store=st), st)
         source = BatchWorkSource(store=st)
         for i in range(BATCH_WORK_UNITS):
             BatchWorkItem(store=st)
@@ -604,7 +599,7 @@ class RemoteTestCase(unittest.TestCase):
 
 
         self.assertEquals(
-            len(log.flushErrors(BrokenException)),
+            len(self.flushLoggedErrors(BrokenException)),
             BATCH_WORK_UNITS)
 
         self.assertEquals(
