@@ -9,7 +9,7 @@ from twisted.python import log, filepath
 
 from axiom import store, item
 from axiom.store import Store
-from axiom.item import Item
+from axiom.item import Item, declareLegacyItem
 from axiom.errors import ChangeRejected
 from axiom.test import itemtest, itemtestmain
 from axiom.attributes import integer, text, inmemory
@@ -162,6 +162,17 @@ class ItemTestCase(unittest.TestCase):
         self.failIf(B == A)
 
 
+    def test_legacyItemComparison(self):
+        """
+        Legacy items with different versions must not compare equal.
+        """
+        legacy1 = declareLegacyItem('test_type', 1, {})
+        legacy2 = declareLegacyItem('test_type', 2, {})
+        self.assertNotEqual(legacy1, legacy2)
+        self.assertEqual(legacy1, legacy1)
+        self.assertEqual(legacy2, legacy2)
+
+
     def testCreateItem(self):
         st = store.Store()
         self.assertRaises(item.CantInstantiateItem, item.Item, store=st)
@@ -340,6 +351,34 @@ class ItemTestCase(unittest.TestCase):
                           1234)
 
 
+    def test_duplicateDefinition(self):
+        """
+        When the same typeName is defined as an item class multiple times in
+        memory, the second definition fails with a L{RuntimeError}.
+        """
+        class X(Item):
+            dummy = integer()
+        try:
+            class X(Item):
+                dummy = integer()
+        except RuntimeError:
+            pass
+        else:
+            self.fail("Duplicate definition should have failed.")
+
+
+    def test_nonConflictingRedefinition(self):
+        """
+        If the python item class associated with a typeName is garbage
+        collected, a new python item class can re-use that type name.
+        """
+        class X(Item):
+            dummy = integer()
+        del X
+        class X(Item):
+            dummy = integer()
+
+
 
 class TestItem(Item):
     """
